from __future__ import division
from string import Template
from parse import expr

op_table = {'+': lambda x, y: x + y,
            '*': lambda x, y: x*y,
            '-': lambda x, y: x - y,
            '/': lambda x, y: x/y,
            '^': lambda x, y: x**y,
            '**': lambda x, y: x**y}

def calc(formula):
    stack = []
    op_stack = []
    pop_next = False
    for token in formula:
        if token in op_table:
            op_stack.append(token)
            pop_next = True
        elif pop_next:
            x = stack.pop()
            stack.append(op_table[op_stack.pop()](x, token))
        else:
            stack.append(token)
    return stack[0]
   
def process(formula):
    todo = []
    for token in formula:
        if isinstance(token, list):
            todo.append(process(token))
        else:
            todo.append(token)
    return calc(todo)

def evaluate(formula, **kwargs):
    if kwargs is not None:
        formula = Template(formula).substitute(**kwargs)
    tokens = expr.parseString(formula)[0].asList()
    return process(tokens)
   

if __name__ == '__main__':
    import sys

    # Assumes input is of the form 'formula', 'kwargs ...'
    var_dict = dict(pair.split(':') for pair in sys.argv[2:])

    print 'result:', evaluate(sys.argv[1], **var_dict)
